'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = createThermostat;

var _eq3ble = require('eq3ble');

var _eq3ble2 = _interopRequireDefault(_eq3ble);

var _mqtt = require('mqtt');

var _mqtt2 = _interopRequireDefault(_mqtt);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function createThermostat(_ref) {
  var Service = _ref.Service,
      Characteristic = _ref.Characteristic;

  return function () {
    function EQ3Thermostat(log, config) {
      var _this = this;

      _classCallCheck(this, EQ3Thermostat);

      this.log = log;
      this.name = config.name;
      this.address = config.address;
      this.discoverTimeout = config.discoverTimeout || 60 * 1000; // 1 minute
      this.connectionTimeout = config.connectionTimeout || 10 * 1000; // 10 seconds
      this.disableBoostSwitch = false;
      this.device = null;
      this.info = null;
      this.isConnected = false;

      this.temperatureDisplayUnits = Characteristic.TemperatureDisplayUnits.CELSIUS;

      this.thermostatService = new Service.Thermostat(this.name);
      this.informationService = new Service.AccessoryInformation();
      this.boostService = new Service.Switch(this.name + ' boost mode');
      this.currentTemperature = null;

      if (config.currentTemperature) {
        this.mqttClient = _mqtt2.default.connect(config.currentTemperature.url, {
          keepalive: 10,
          clientId: 'mqttjs_'.concat(Math.random().toString(16).substr(2, 8)),
          protocolId: 'MQTT',
          protocolVersion: 4,
          clean: true,
          reconnectPeriod: 1000,
          connectTimeout: 30 * 1000,
          will: {
            topic: 'WillMsg',
            payload: 'Connection Closed abnormally..!',
            qos: 0,
            retain: false
          },
          username: config.currentTemperature.username,
          password: config.currentTemperature.password,
          rejectUnauthorized: false
        });

        this.mqttClient.on('connect', function () {
          _this.mqttClient.subscribe(config.currentTemperature.topic);
        });

        this.mqttClient.on('message', function (topic, message) {
          var mqttData = JSON.parse(message);
          if (mqttData === null) {
            return null;
          }
          _this.currentTemperature = parseFloat(mqttData);
          _this.thermostatService.setCharacteristic(Characteristic.CurrentTemperature, _this.currentTemperature);
          return _this.currentTemperature;
        });
      }

      this.boostService.setCharacteristic(Characteristic.Name, 'Boost Mode').getCharacteristic(Characteristic.On).on('get', this.execAfterConnect.bind(this, this.getBoost.bind(this))).on('set', this.execAfterConnect.bind(this, this.setBoost.bind(this)));

      this.informationService.setCharacteristic(Characteristic.Manufacturer, 'eq-3').setCharacteristic(Characteristic.Model, 'CC-RT-BLE');

      this.thermostatService.getCharacteristic(Characteristic.CurrentHeatingCoolingState).on('get', this.execAfterConnect.bind(this, this.getCurrentHeatingCoolingState.bind(this)));

      this.thermostatService.getCharacteristic(Characteristic.TargetHeatingCoolingState).on('get', this.execAfterConnect.bind(this, this.getTargetHeatingCoolingState.bind(this))).on('set', this.execAfterConnect.bind(this, this.setTargetHeatingCoolingState.bind(this)));

      if (this.mqttClient) {
        this.thermostatService.getCharacteristic(Characteristic.CurrentTemperature).on('get', this.getCurrentTemperature.bind(this));
      } else {
        this.thermostatService.getCharacteristic(Characteristic.CurrentTemperature).on('get', this.execAfterConnect.bind(this, this.getTargetTemperature.bind(this)));
      }

      this.thermostatService.getCharacteristic(Characteristic.TargetTemperature).on('get', this.execAfterConnect.bind(this, this.getTargetTemperature.bind(this))).on('set', this.execAfterConnect.bind(this, this.setTargetTemperature.bind(this)));

      this.thermostatService.getCharacteristic(Characteristic.TemperatureDisplayUnits).on('get', this.execAfterConnect.bind(this, this.getTemperatureDisplayUnits.bind(this))).on('set', this.execAfterConnect.bind(this, this.setTemperatureDisplayUnits.bind(this)));

      this.thermostatService.getCharacteristic(Characteristic.HeatingThresholdTemperature).on('get', this.execAfterConnect.bind(this, this.getTargetTemperature.bind(this)));

      this.discoverPromise = this.discover().catch(function (err) {
        _this.log(err);
      });
    }

    _createClass(EQ3Thermostat, [{
      key: 'discover',
      value: function discover() {
        var _this2 = this;

        return new Promise(function (resolve, reject) {
          _this2.log('discovering thermostat (' + _this2.address + ')');
          var discoverTimeout = setTimeout(function () {
            _this2.discoverPromise = null;
            _this2.log('cannot discover thermostat (' + _this2.address + ')');
            reject(new Error('discovering thermostat timed out (' + _this2.address + ')'));
          }, _this2.discoverTimeout);
          _eq3ble2.default.discoverByAddress(_this2.address, function (device) {
            clearTimeout(discoverTimeout);
            _this2.device = device;
            _this2.log('discovered thermostat (' + _this2.address + ')');
            resolve();
          }, function (err) {
            _this2.discoverPromise = null;
            _this2.log('cannot discover thermostat (' + _this2.address + '): ' + err);
            reject(new Error('discovering thermostat (' + _this2.address + ') resulted in error ' + err));
          });
        });
      }
    }, {
      key: 'connect',
      value: function connect() {
        var _this3 = this;

        this.discoverPromise = this.discoverPromise || this.discover();
        return this.discoverPromise.then(function () {
          _this3.connectionPromise = _this3.connectionPromise || new Promise(function (resolve, reject) {
            clearTimeout(_this3.timeout);
            if (_this3.isConnected) {
              _this3.connectionPromise = null;
              resolve();
              return;
            }
            _this3.log('connecting to thermostat (' + _this3.address + ')');
            _this3.connectTimeout = setTimeout(function () {
              _this3.log('connection to thermostat timed out (' + _this3.address + ')');
              _this3.connectionPromise = null;
              _this3.isConnected = false;
              _this3.discoverPromise = null;
              reject(new Error('connection to thermostat timed out (' + _this3.address + ')'));
            }, _this3.discoverTimeout);
            _this3.device.connectAndSetup().then(function () {
              clearTimeout(_this3.connectTimeout);
              _this3.log('connected to thermostat (' + _this3.address + ')');
              _this3.connectionPromise = null;
              _this3.isConnected = true;
              resolve();
            }, function (err) {
              clearTimeout(_this3.connectTimeout);
              _this3.log('cannot connect to thermostat (' + _this3.address + ')');
              _this3.connectionPromise = null;
              _this3.isConnected = false;
              _this3.discoverPromise = null;
              reject(err || new Error('cannot connect to thermostat (' + _this3.address + ')'));
            });
          });
          return _this3.connectionPromise;
        });
      }
    }, {
      key: 'disconnect',
      value: function disconnect() {
        if (this.device) this.device.disconnect();
        this.isConnected = false;
        this.connectionPromise = null;
        this.log('disconnected from thermostat (' + this.address + ')');
      }
    }, {
      key: 'startDisconnectTimeout',
      value: function startDisconnectTimeout() {
        clearTimeout(this.timeout);
        this.timeout = setTimeout(this.disconnect.bind(this), this.connectionTimeout);
      }
    }, {
      key: 'execAfterConnect',
      value: function execAfterConnect(fn) {
        var _this4 = this;

        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        this.connect().then(function () {
          fn.apply(undefined, args);
          _this4.startDisconnectTimeout();
        }, function (err) {
          err.isError = true;
          fn.apply(undefined, _toConsumableArray(args.concat(err)));
        });
      }
    }, {
      key: 'getDeviceInfo',
      value: function getDeviceInfo() {
        var _this5 = this;

        this.log('getting thermostat info (' + this.address + ')');
        return this.device.getInfo().then(function (info) {
          _this5.log('got thermostat info (' + _this5.address + ')');
          return info;
        });
      }
    }, {
      key: 'getCachedInfo',
      value: function getCachedInfo() {
        var _this6 = this;

        return new Promise(function (resolve, reject) {
          if (_this6.info) {
            resolve(_this6.info);
            return;
          }
          _this6.infoPromise = _this6.infoPromise || _this6.getDeviceInfo();
          _this6.infoPromise.then(function (info) {
            _this6.info = info;
            _this6.infoPromise = null;
            resolve(_this6.info);
            setTimeout(function () {
              _this6.info = null;
            }, 250);
          }, reject);
        });
      }
    }, {
      key: 'getBoost',
      value: function getBoost(callback) {
        for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.getCachedInfo().then(function (_ref2) {
          var boost = _ref2.boost;

          callback(null, boost);
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'getCurrentHeatingCoolingState',
      value: function getCurrentHeatingCoolingState(callback) {
        for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
          args[_key3 - 1] = arguments[_key3];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.getCachedInfo().then(function (_ref3) {
          var valvePosition = _ref3.valvePosition;

          if (valvePosition) {
            callback(null, Characteristic.CurrentHeatingCoolingState.HEAT);
          } else {
            callback(null, Characteristic.CurrentHeatingCoolingState.OFF);
          }
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'getTargetHeatingCoolingState',
      value: function getTargetHeatingCoolingState(callback) {
        for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
          args[_key4 - 1] = arguments[_key4];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.getCachedInfo().then(function (_ref4) {
          var targetTemperature = _ref4.targetTemperature,
              status = _ref4.status;

          if (targetTemperature <= 4.5) {
            callback(null, Characteristic.TargetHeatingCoolingState.OFF);
          } else if (targetTemperature >= 30 || status.boost) {
            callback(null, Characteristic.TargetHeatingCoolingState.HEAT);
          } else if (status.manual) {
            callback(null, Characteristic.TargetHeatingCoolingState.COOL);
          } else {
            callback(null, Characteristic.TargetHeatingCoolingState.AUTO);
          }
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'getTargetTemperature',
      value: function getTargetTemperature(callback) {
        for (var _len5 = arguments.length, args = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
          args[_key5 - 1] = arguments[_key5];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.getCachedInfo().then(function (_ref5) {
          var targetTemperature = _ref5.targetTemperature;

          callback(null, targetTemperature < 10 ? 10 : targetTemperature);
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'getCurrentTemperature',
      value: function getCurrentTemperature(callback) {
        this.log(this.name, ' - MQTT : ', this.currentTemperature);
        callback(null, this.currentTemperature);
      }
    }, {
      key: 'getTemperatureDisplayUnits',
      value: function getTemperatureDisplayUnits(callback) {
        for (var _len6 = arguments.length, args = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
          args[_key6 - 1] = arguments[_key6];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return callback(null, this.temperatureDisplayUnits);
      }
    }, {
      key: 'setBoost',
      value: function setBoost(value, callback) {
        for (var _len7 = arguments.length, args = Array(_len7 > 2 ? _len7 - 2 : 0), _key7 = 2; _key7 < _len7; _key7++) {
          args[_key7 - 2] = arguments[_key7];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.device.setBoost(value).then(function () {
          return callback();
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'setTemperatureDisplayUnits',
      value: function setTemperatureDisplayUnits(value, callback) {
        for (var _len8 = arguments.length, args = Array(_len8 > 2 ? _len8 - 2 : 0), _key8 = 2; _key8 < _len8; _key8++) {
          args[_key8 - 2] = arguments[_key8];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        this.temperatureDisplayUnits = value;
        return callback();
      }
    }, {
      key: 'setTargetTemperature',
      value: function setTargetTemperature(value, callback) {
        for (var _len9 = arguments.length, args = Array(_len9 > 2 ? _len9 - 2 : 0), _key9 = 2; _key9 < _len9; _key9++) {
          args[_key9 - 2] = arguments[_key9];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        return this.device.setTemperature(value).then(function () {
          return callback();
        }, function (deviceErr) {
          return callback(deviceErr);
        });
      }
    }, {
      key: 'setTargetHeatingCoolingState',
      value: function setTargetHeatingCoolingState(value, callback) {
        for (var _len10 = arguments.length, args = Array(_len10 > 2 ? _len10 - 2 : 0), _key10 = 2; _key10 < _len10; _key10++) {
          args[_key10 - 2] = arguments[_key10];
        }

        var lastArg = args && args[args.length - 1];
        if (lastArg.isError) return callback(lastArg);
        switch (value) {
          case Characteristic.TargetHeatingCoolingState.OFF:
            return this.device.turnOff().then(function () {
              return callback();
            }, function (deviceErr) {
              return callback(deviceErr);
            });
          case Characteristic.TargetHeatingCoolingState.HEAT:
            return this.device.turnOn().then(function () {
              return callback();
            }, function (deviceErr) {
              return callback(deviceErr);
            });
          case Characteristic.TargetHeatingCoolingState.AUTO:
            return this.device.automaticMode().then(function () {
              return callback();
            }, function (deviceErr) {
              return callback(deviceErr);
            });
          case Characteristic.TargetHeatingCoolingState.COOL:
            return this.device.manualMode().then(function () {
              return callback();
            }, function (deviceErr) {
              return callback(deviceErr);
            });
          default:
            return callback('Unsupported mode');
        }
      }
    }, {
      key: 'getServices',
      value: function getServices() {
        return [this.informationService, this.thermostatService, !this.disableBoostSwitch && this.boostService];
      }
    }]);

    return EQ3Thermostat;
  }();
}